/*
**  SLOPE_FOR: C-XSC classes for one- and multidimensional (interval)
**             slope arithmetic in forward mode
**
**  Copyright (C) 1999-2001 M. Braeuer, W. Hofschuster, W. Kraemer
**                          Wiss. Rechnen/Softwaretechnologie
**                          Universitaet Wuppertal, Germany   
**
**  This library is free software; you can redistribute it and/or
**  modify it under the terms of the GNU Library General Public
**  License as published by the Free Software Foundation; either
**  version 2 of the License, or (at your option) any later version.
**
**  This library is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
**  Library General Public License for more details.
**
**  You should have received a copy of the GNU Library General Public
**  License along with this library; if not, write to the Free
**  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "helpfuncs.h"

///////////////////////////////////////////////////////////////
//
//Implementierung der globalen Hilfsfunktionen aus helpfuncs.h
//
///////////////////////////////////////////////////////////////


namespace cxsc {

int even(int n){return !(n%2);};

//------------------------------------------------------------------------

interval h_abs(const real& x1,const interval& x2)
{
 interval res;
 if(!(_interval(x1)<=x2))
   {
    res=(abs(_interval(x1))-abs(x2))/(_interval(x1)-x2);
   }
 else res=_interval(-1.0,1.0);

 return res;
}

//------------------------------------------------------------------------

interval h_max(const interval& x1, const interval& x2)
{
 interval res;
 res=_interval(max(Inf(x1),Inf(x2)),max(Sup(x1),Sup(x2)));
 return res;
}

//------------------------------------------------------------------------

interval chi(const interval& s, const interval& q , const interval& r)
{
 if (Sup(s)<0) return q;
 if (Inf(s)>0) return r;
 return q|r; 
}

//Hilfsfunktionen zur Slopebestimmung bei Standardfunktionen
//------------------------------------------------------------------------

int s_1_down(const interval& x, const interval& c, real& res, interval(*f)(const interval& ))
{
 interval _x=_interval(Inf(x));
 interval _c=_interval(Inf(c));

 interval h= _x-_c; 

 if(_interval(0.0)<=h) return 0;
  else
   {
     interval _fx=f(_x);
     interval _fc=f(_c);
     interval i_res=(_fx-_fc)/h;
     res = Inf(i_res);
     return 1;
   }
}

//------------------------------------------------------------------------

int s_1_up(const interval& x, const interval& c, real& res, interval(*f)(const interval& ))
{
 interval _x=_interval(Inf(x));
 interval _c=_interval(Inf(c));

 interval h= _x-_c; 

 if(_interval(0.0)<=h) return 0;
  else
   {     
     interval _fx=f(_x);
     interval _fc=f(_c);
     interval i_res=(_fx-_fc)/h;
     res = Sup(i_res);
     return 1;
   }
}

//------------------------------------------------------------------------

int s_2_down(const interval& x, const interval& c,real& res, interval(*f)(const interval& ))
{
 interval x_=_interval(Sup(x));
 interval c_=_interval(Sup(c));

 interval h= x_-c_; 

 if(_interval(0.0)<=h) return 0;
  else
   {
     interval fx_=f(x_);
     interval fc_=f(c_);
     interval i_res=(fx_-fc_)/h;
     res = Inf(i_res);
     return 1;
   }
}

//------------------------------------------------------------------------

int s_2_up(const interval& x, const interval& c, real& res, interval(*f)(const interval& ))
{
 interval x_=_interval(Sup(x));
 interval c_=_interval(Sup(c));

 interval h= x_-c_; 

 if(_interval(0.0)<=h) return 0;
  else
   {
     interval fx_=f(x_);
     interval fc_=f(c_);
     interval i_res=(fx_-fc_)/h;
     res = Sup(i_res);
     return 1;
   }
}




//Fuer Funktionen mit zwei Argumenten
//------------------------------------------------------------------------

int s_1_down(const interval& x, const interval& c, int n, real& res,
	     interval(*f)(const interval& , const int ))
{
 interval _x=_interval(Inf(x));
 interval _c=_interval(Inf(c));

 interval h= _x-_c; 

 if(_interval(0.0)<=h) return 0;
  else
   {
     interval _fx=f(_x,n);
     interval _fc=f(_c,n);
     interval i_res=(_fx-_fc)/h;

     res = Inf(i_res);
     return 1;
   }
}

//------------------------------------------------------------------------

int s_1_up(const interval& x, const interval& c, int n, real& res,
	   interval(*f)(const interval& , const int ))
{
 interval _x=_interval(Inf(x));
 interval _c=_interval(Inf(c));

 interval h= _x-_c; 

 if(_interval(0.0)<=h) return 0;
  else
   {     
     interval _fx=f(_x,n);
     interval _fc=f(_c,n);
     interval i_res=(_fx-_fc)/h;
     res = Sup(i_res);
     return 1;
   }
}

//------------------------------------------------------------------------

int s_2_down(const interval& x, const interval& c, int n,real& res,
	     interval(*f)(const interval& , const int ))
{
 interval x_=_interval(Sup(x));
 interval c_=_interval(Sup(c));

 interval h= x_-c_; 

 if(_interval(0.0)<=h) return 0;
  else
   {
     interval fx_=f(x_,n);
     interval fc_=f(c_,n);
     interval i_res=(fx_-fc_)/h;
     res = Inf(i_res);
     return 1;
   }
}

//------------------------------------------------------------------------

int s_2_up(const interval& x, const interval& c, int n, real& res,
	   interval(*f)(const interval& , const int ))
{
 interval x_=_interval(Sup(x));
 interval c_=_interval(Sup(c));

 interval h= x_-c_; 

 if(_interval(0.0)<=h) return 0;
  else
   {
     interval fx_=f(x_,n);
     interval fc_=f(c_,n);
     interval i_res=(fx_-fc_)/h;
     res = Sup(i_res);
     return 1;
   }
}

} // namespace cxsc
