/*
**  SLOPE_FOR: C-XSC classes for one- and multidimensional (interval)
**             slope arithmetic in forward mode
**
**  Copyright (C) 1999-2001 M. Braeuer, W. Hofschuster, W. Kraemer
**                          Wiss. Rechnen/Softwaretechnologie
**                          Universitaet Wuppertal, Germany   
**
**  This library is free software; you can redistribute it and/or
**  modify it under the terms of the GNU Library General Public
**  License as published by the Free Software Foundation; either
**  version 2 of the License, or (at your option) any later version.
**
**  This library is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
**  Library General Public License for more details.
**
**  You should have received a copy of the GNU Library General Public
**  License along with this library; if not, write to the Free
**  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

//////////////////////////////////////////////////////////////////////////
//
// Headerfile multi_dim_slope.h fuer mehrdimensionale 
// Intervallsteigungsarithmetik
//
//////////////////////////////////////////////////////////////////////////

#ifndef _MULTIDIMSLOPE_H
#define _MULTIDIMSLOPE_H

//cxsc headers
#include "interval.hpp"
#include "imath.hpp"
#include "ivector.hpp"

//C++ standard headers
#include <iostream>
#include <stdlib.h>

//einbinden der globalen Hilfsfunktionen
#include "helpfuncs.h"

//debug flag
#define debug

namespace cxsc {

/*----------------------------------------------------------------------
Compiler: g++/gcc version 2.7.2.1
------------------------------------------------------------------------*/

//////////////////////////////////////////////////////////////////////////
//
//Definitionen der Klassen
//
//////////////////////////////////////////////////////////////////////////
/*-----------------------------------------------------------------------

class slope  automatische Berechnung mehrdimensionaler Intervallsteigungen
             angelehnt an PASCAL-XSC Version von Dr. D. Ratz (slp_ari.p)

  Elemente: 
            int dim ...................... Dimension der Unabhaeng.
            interval  f_x ............... 1.Intervalkomponente (Fkt.wert)
            interval  f_c ............... 2.Intervalkomponente (Fkt.wert)
	    ivector   f_s ............... Steigungvektor

            static int calc_slope ....... falls 1 werden Steigungen  be-
	                                  rechnet; default ist 1

  Elementfunktionen:

            siehe Implementation 

Implementationsfile: multi_dim_slope.cpp

-----------------------------------------------------------------------*/
class slope{
 private: 
  int dim;
  interval f_x; //f(X)
  interval f_c; //f(C)
  ivector  f_s; //Intervallvektor mit Lb(f_s)=1 Ub(f_s)=dim

  static int calc_slope;

 public:

  static void set_calc_slope();
  static void unset_calc_slope();
  static int get_calc_slope();

  slope();
  slope(int );
  slope(const slope& );

  slope& operator=(const slope&);

  friend slope var_slope(int, int,const  real& );
  friend slope var_slope(int, int, const interval& );
  friend slope var_slope(int, int,const  interval&,const  interval& );
  friend slope var_slope(int, int,const  interval&,const  real& );

  friend slope const_slope(int,const  real& );
  friend slope const_slope(int,const  interval& );

  friend slope init_mid_var(int, int,const  interval& );
  
  friend void print_slope(const slope&);

  friend int get_dim(const slope& );
  friend interval get_fx(const slope& );
  friend interval get_fc(const slope& );
  friend ivector get_fs(const slope& ); 

  friend slope operator-(const slope& ); 

  friend slope operator-(const slope& ,const slope& );
  friend slope operator+(const slope& ,const slope& );
  friend slope operator*(const slope& ,const slope& );
  friend slope operator/(const slope& ,const slope& );

  friend slope operator-(const slope& ,const interval& );
  friend slope operator+(const slope& ,const interval& );
  friend slope operator*(const slope& ,const interval& );
  friend slope operator/(const slope& ,const interval& ); 

  friend slope operator-(const interval& ,const slope& );
  friend slope operator+(const interval& ,const slope& );
  friend slope operator*(const interval& ,const slope& );
  friend slope operator/(const interval& ,const slope& ); 

  friend slope operator-(const slope& ,const real& );
  friend slope operator+(const slope& ,const real& );
  friend slope operator*(const slope& ,const real& );
  friend slope operator/(const slope& ,const real& ); 

  friend slope operator-(const real& ,const slope& );
  friend slope operator+(const real& ,const slope& );
  friend slope operator*(const real& ,const slope& );
  friend slope operator/(const real& ,const slope& ); 

  friend slope sqr(const slope&);
  friend slope sqrt(const slope&);
  friend slope sqrt(const slope&, int);

  friend slope power(const slope&, int);
  friend slope pow(const slope&, const interval& );

  friend slope ln(const slope&); 
  friend slope exp(const slope&); 

  friend slope sin(const slope&); 
  friend slope cos(const slope&);
  friend slope tan(const slope&);
  friend slope cot(const slope&);

   friend slope asin(const slope&); 
   friend slope acos(const slope&);
   friend slope atan(const slope&);
   friend slope acot(const slope&);

   friend slope sinh(const slope&); 
   friend slope cosh(const slope&);
   friend slope tanh(const slope&);
   friend slope coth(const slope&);

   friend slope asinh(const slope&); 
   friend slope acosh(const slope&);
   friend slope atanh(const slope&);
   friend slope acoth(const slope&);

   friend slope abs(const slope&);
   friend slope max(const slope&, const slope&);
   friend slope min(const slope&, const slope&);
   friend slope chi(const slope&, const slope&, const slope&);
};

/*-----------------------------------------------------------------------

Klasse slope_vector  Vektor fuer Objekte vom Typ slope
  Elemente: 
            int      dim ............... Dimension des Vektors vom Typ
	                                 slope_vector
	    int      lb ................ lower bound
            int      dim ............... upper bound
            int      dim_slopes ........ Dimension der Vektors f_s der
	                                 slopes
	    slope*   sv ................ Zeiger auf Vektor von slopes

  Elementfunktionen:

            siehe Implementation 

Implementationsfile: multi_dim_slope.cpp

-----------------------------------------------------------------------*/ 
class slope_vector{

 private:
  int dim;
  int lb;
  int ub;
  int dim_slopes;
  slope* sv;

 public:
 
  slope_vector();
  slope_vector(int, int, int );

  slope_vector(const slope_vector& );
  ~slope_vector();

  slope_vector& operator=(const slope_vector& s);
  slope& operator[](int n);
  slope& operator[](int n) const;
  
  friend int Lb(const slope_vector& x);
  friend int Ub(const slope_vector& x);

  friend slope_vector InitVar(const ivector& );
  friend slope_vector InitVar(const rvector& );
  friend slope_vector InitVar(const ivector& , const ivector& ); 
  friend slope_vector InitVar(const ivector& , const rvector& ); 

  friend slope_vector Init_mid_Var(const ivector& );

//#ifdef debug
  friend void print_slope_vector(const slope_vector& s);
//#endif
};

void feval(slope (*f)(const slope_vector& ), const ivector&, interval& );
void fseval(slope (*f)(const slope_vector& ),const  ivector&, 
	   interval& , ivector& );

} //namespace cxsc

#endif
